#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/


in float Height;
in vec2 textureCoord_te;
in vec4 n;
in vec4 position;

uniform sampler2D meadowTex;
uniform sampler2D flowerTex;
uniform vec3 materialCoefficients;
uniform float specularAlpha;
uniform vec3 pos;
uniform vec3 camera_pos;

uniform struct PointLight{
    vec3 color;
    vec3 position;
    vec3 attenuation;
} pointL;

out vec4 outColor;



vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

//from https://www.shadertoy.com/view/Ddlyzr for rgb to hsv and other way convertion
vec3 rgb2hsv(vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}
vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main() {	
    float levels = 12;
    vec3 unitNormal = normalize(n.xyz);
    vec3 eyeDir = normalize(-position).xyz;

    vec3 totalDiffuse = vec3 (0, 0, 0);
    vec3 totalSpecular = vec3(0,0, 0);

    float distance = length(position.xyz - pointL.position.xyz);
    float attFactor = pointL.attenuation.x + pointL.attenuation.y * distance + pointL.attenuation.z * distance * distance;
    vec3 unitLightVector = normalize(position.xyz - pointL.position);
    
    float nDotl = dot(unitNormal, unitLightVector);
    float brightness = max(nDotl, 0.0);
    float level = floor(brightness * levels);
    brightness = level/levels; 

    vec3 lightDirection = -unitLightVector;
    vec3 reflectedLightDirection = reflect(lightDirection, unitNormal);
    float specularFactor = dot(reflectedLightDirection, eyeDir);
    specularFactor = max(specularFactor, 0.0);
    totalDiffuse = totalDiffuse + (brightness*0.6 * pointL.color/pointL.attenuation);

    totalDiffuse = max(totalDiffuse, 0.1);

    float h = (Height + 16)/64.0f;
	vec4 FragColor = vec4(h, h, h, 1.0);
	FragColor = texture(meadowTex, textureCoord_te);
    vec3 FragColorhsv = rgb2hsv(FragColor.rgb);
    //thin matrix tutorial cel shading
    level = floor(FragColorhsv.z * levels);
	FragColorhsv.z = (level / levels);
    FragColor = vec4(hsv2rgb(FragColorhsv), 1.0);

    
    
    outColor = vec4(totalDiffuse*0.3, 1.0) * FragColor;
}

