#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	float d = length(- dirL.direction);
	float att = 1.0f * pointL.attenuation.x + d * pointL.attenuation.y + d * d * pointL.attenuation.z;


	vec3 lightDir = normalize(pointL.position - vert.position_world);
	// ambient
    vec3 ambient = materialCoefficients.x * pointL.color;
    
    // diffuse
	float diff = max(dot(n, lightDir), 0.0);	

	// cel shading
    float levels = 10.0;
    float level = floor(diff * levels);
	diff = level / levels;

	vec3 totalDiffuse = diff * pointL.color * materialCoefficients.y /att;

    // specular
    vec3 viewDir = normalize(camera_world - vert.position_world);
   
    vec3 halfwayDir = normalize(lightDir + viewDir);  
    float dampedFactor = pow(max(dot(n, halfwayDir), 0.0), specularAlpha);
    vec3 specular =  dampedFactor * pointL.color * materialCoefficients.z /att;   
	
    vec3 light = ambient + totalDiffuse + specular;

	color = vec4(light, 1);

}

